/*
 * msmtp.c
 *
 * This file is part of the msmtp package, a very simple programm that 
 * works as an "SMTP plugin" for Mutt and possibly other MUAs.
 *
 * Copyright (C) 2000, 2003	Martin Lambers <marlam@web.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>
#include <sysexits.h>

#include "conffile.h"
#include "merror.h"
#include "list.h"
#include "esmtp.h"
#include "xmalloc.h"

#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#else
#include "getopt.h"
#endif /* HAVE_GETOPT_LONG */

/* The name of this program */
char *prgname;
/* The version number */
#define VERSION "0.5.1"
/* The name of the configuration file */
#define CONFFILE ".msmtprc"


/*
 * The main function processes the options.
 * It returns values from sysexits.h (like sendmail does).
 */

int main(int argc, char *argv[])
{
    /* the configuration */
    int print_version;
    int print_help;
    int execute;
    char *conffile;
    char *account_id;
    list_t *account_list;
    account_t *account;
    /* the recipients of the mail */
    int rcptc;
    char **rcptv;
    /* misc */
    int error_code;
    char *p;
    /* commandline processing with getopt_long */
    int c;
    int option_index;
    struct option long_options[] =
    {
	{ "version", no_argument, 0, 'v' },
	{ "help",    no_argument, 0, 'h' },
	{ "file",    required_argument, 0, 'f' },
	{ "account", required_argument, 0, 'a' },
	{ 0, 0, 0, 0 }
    };

    
    /* process the command line */
    prgname = strrchr(argv[0], '/');
    if (!prgname)
    {
	prgname = argv[0];
    }
    else
    {
	prgname++;
    }
    
    error_code = 0;
    print_version = 0;
    print_help = 0;
    execute = 1;
    conffile = NULL;
    account_id = NULL;
    for (;;)
    {
	c = getopt_long(argc, argv, "vhf:a:", long_options, &option_index);
	if (c == -1)
	{
	    break;
	}
	switch(c)
	{
	    case 'v':
		print_version = 1;
		execute = 0;
		break;

	    case 'h':
		print_help = 1;
		execute = 0;
		break;

	    case 'f':
		conffile = xstrdup(optarg);
		break;

	    case 'a':
		account_id = xstrdup(optarg);
		break;

	    default:
		error_code = 1;
		break;
	}
	if (error_code)
	{
	    break;
	}
    }
    if (error_code)
    {
	return EX_USAGE;
    }

    if (print_version)
    {
	printf("This is %s version %s\n", prgname, VERSION);
    }
    if (print_help)
    {
	printf("Usage: %s <options> [--] recipient1 [more recipients]\n"
		"Options are:\n"
		"--version        print version and exit\n"
		"--help           print help and exit\n"
		"--file=filename  use 'filename' as configuration file\n"
		"--account=id     use the account 'id' instead of 'default'\n",
		prgname);
    }
    if (!execute)
    {
	return EX_OK;
    }
    
    /* get the list of recipients */
    rcptc = argc - optind;
    rcptv = &(argv[optind]);
    if (rcptc == 0)
    {
	add_error_line("no recipients given");
	print_error_lines(stderr);
	return EX_USAGE;
    }

    /* read the configuration file */
    if (!conffile)
    {
	if (!(p = getenv("HOME")))
	{
	   add_error_line("no environment variable HOME");
	   print_error_lines(stderr);
	   return EX_NOINPUT;
	}
	conffile = (char *)xmalloc((strlen(p) + strlen(CONFFILE) + 2) 
		* sizeof(char));
	strcpy(conffile, p);
	strcat(conffile, "/" CONFFILE);
    }
    if ((error_code = get_conf(conffile, &account_list)) != 0)
    {
	add_error_line("could not use configuration file `%s'", conffile);
	print_error_lines(stderr);
	free(conffile);
	free(account_id);
	if (error_code == 1)
	{
	    return EX_IOERR;
	}
	else
	{
	    return EX_DATAERR;
	}
    }

    /* get the account data */
    if (!account_id)
    {
	account_id = xstrdup("default");
    }
    if (!(account = find_account(account_list, account_id)))
    {
	add_error_line("no account `%s' found in configuration file `%s'",
		account_id, conffile);
	print_error_lines(stderr);
	free(conffile);
	free(account_id);
	return EX_DATAERR;
    }

    /* DEBUG
    printf("account = %s from %s\n"
	    "host = %s\n"
	    "port = %d\n"
	    "from = %s\n"
	    "auth = %d\n"
	    "user = %s\n"
	    "pass = %s\n"
	    "tls  = %d\n"
	    "nst  = %d\n"
	    "nkb  = %d\n",
	    account->id, conffile, 
	    account->host, 
	    account->port, 
	    account->from,
	    account->auth_method, 
	    account->username, 
	    account->password, 
	    account->tls_method, 
	    account->nostarttls, 
	    account->keepbcc);
    */
    
    /* send the mail */
    if ((error_code = smtp_send(
		    account->host, 
		    account->port, 
		    account->from,
		    account->auth_method, 
		    account->username, 
		    account->password, 
		    account->tls_method, 
		    account->nostarttls, 
		    account->keepbcc,
		    rcptc, rcptv, 
		    stdin)) != EX_OK)
    {
	add_error_line("could not send mail [account: `%s', file: `%s']",
		account_id, conffile);
	print_error_lines(stderr);
    }

    /* clean up */
    free(conffile);
    free(account_id);
    list_xfree(account_list, _account_free);

    return error_code;
}
